
(* signature for structure implementing MinML 
   using a named variable representation *)

signature MINML =
sig

  datatype typ =
      INT
    | BOOL
    | ARROW of typ * typ

  datatype primop = Equal | Plus | Minus | Times | Negate

  type var = string
  val varEqual : var * var -> bool

  datatype exp =
      Int of int

    | Bool of bool

    (* if ec then et else ef fi 
       If(ec, et, ef) *)
    | If of exp * exp * exp

    | Primop of primop * exp list

  (* fun f (x : td) : tr is e end
     Fun(f, x, td, tr, e) *)
    | Fun of var * var * typ * typ * exp

  (* e1 e2
     Apply(e1, e2) *)
    | Apply of exp * exp

    | Var of var

  (* given a primop, get its argument and result types *)
  val typeOfPrimop : primop -> typ list * typ

  (* given a primop and some arguments, try to apply it *)
  val evalPrimop : primop * exp list -> exp option

end


(* signature for structure implementing MinML 
   using a DeBruijn representation *)

signature DBMINML =
sig

  datatype typ =
      INT
    | BOOL
    | ARROW of typ * typ

  datatype primop = Equal | Plus | Minus | Times | Negate

  type var = int
  val varEqual : var * var -> bool

  datatype exp =
      Int of int

    | Bool of bool

    (* if ec then et else ef fi 
       If(ec, et, ef) *)
    | If of exp * exp * exp

    | Primop of primop * exp list

  (* fun f (x : td) : tr is e end
     Fun(f, x, td, tr, e) *)
    | Fun of typ * typ * exp

  (* e1 e2
     Apply(e1, e2) *)
    | Apply of exp * exp

    | Var of var

  (* given a primop, get its argument and result types *)
  val typeOfPrimop : primop -> typ list * typ

  (* given a primop and some arguments, try to apply it *)
  val evalPrimop : primop * exp list -> exp option

end
